%bcond_with testsuite

%ifarch x86_64 aarch64 ppc64le
%global with_dyninst 1
%global dyninst_config --with-dyninst
%else
%global with_dyninst 0
%global dyninst_config --without-dyninst
%endif

%ifarch x86_64 aarch64
%global with_crash 1
%global crash_config --enable-crash
%else
%global with_crash 0
%global crash_config --disable-crash
%endif

%ifarch x86_64
%global with_mokutil 1
%global with_openssl 1
%else
%global with_mokutil 0
%global with_openssl 0
%endif

%global dracutstap %{_prefix}/lib/dracut/modules.d/99stap
%global dracutbindir %{_bindir}
%{!?_rpmmacrodir: %global _rpmmacrodir %{_rpmconfigdir}/macros.d}
%global __brp_mangle_shebangs_exclude_from .stp$
%global py_auto_byte_compile 0

%global systemtap_runtime_sysusers \
# See systemd-sysusers(8) sysusers.d(5) for details.\
\
g     stapusr  156\
g     stapsys  157\
g     stapdev  158

%global systemtap_server_sysusers \
# See systemd-sysusers(8) sysusers.d(5) for details.\
\
g     stap-server  -\
u     stap-server  -      "systemtap compiler server"   /var/lib/stap-server   /sbin/nologin\
m     stap-server stap-server

%if %{with testsuite}
%global systemtap_testsuite_sysusers \
# See systemd-sysusers(8) sysusers.d(5) for details.\
\
u     stapusr  -          "systemtap testsuite user"    /   /sbin/nologin\
u     stapsys  -          "systemtap testsuite user"    /   /sbin/nologin\
u     stapdev  -          "systemtap testsuite user"    /   /sbin/nologin\
m     stapusr  stapusr\
m     stapsys  stapusr\
m     stapsys  stapsys\
m     stapdev  stapusr\
m     stapdev  stapdev
%endif

Summary:		System-wide probe/trace tool
Name:			systemtap
Version:		4.7
Release: 		3%{?dist}
License: 		GPLv2+
URL:     		http://sourceware.org/systemtap/
Source0:		https://sourceware.org/systemtap/ftp/releases/%{name}-%{version}.tar.gz 

BuildRequires:	make gcc-c++ cpio systemd-rpm-macros systemd emacs python3
BuildRequires:	pkgconfig(nss) pkgconfig(avahi-client)
BuildRequires:	pkgconfig(json-c) pkgconfig(ncurses) 
BuildRequires:	gettext-devel sqlite-devel rpm-devel readline-devel
BuildRequires:	python3-devel python3-setuptools
BuildRequires:	sqlite-devel > 3.7
BuildRequires:	elfutils-devel >= 0.179
BuildRequires:	/usr/bin/pathfix.py
%if %{with_dyninst}
BuildRequires:	dyninst-devel >= 10.0
BuildRequires:	pkgconfig(libselinux)
%endif
%if %{with_crash}
BuildRequires:	crash-devel zlib-devel
%endif
Requires:		systemtap-client = %{version}-%{release}
Requires:		systemtap-devel = %{version}-%{release}

%description
Systemtap is a tool that allows developers and administrators to write 
and reuse simple scripts to deeply examine the activities of a live Linux 
system. Data may be extracted, filtered, and summarized quickly and safely, 
to enable diagnoses of complex performance or functional problems.

%package server
Summary:		Systemtap compile server
License:		GPLv2+
BuildRequires:	nss-devel avahi-devel
Requires:		systemtap-devel = %{version}-%{release}
Requires: 		nss coreutils zip unzip systemd
%if %{with_openssl}
Requires:		openssl
%endif
Requires(pre):	shadow-utils

%description server
A systemtap compile server listens for connections from stap clients
on a secure SSL network port and accepts requests to run the stap front 
end. Each server advertises its presence and configuration on the local 
network using mDNS (avahi) allowing for automatic detection by clients.

%package client
Summary:		Client for developing systemtap scripts
License: 		GPLv2+
Requires:		coreutils grep sed unzip zip openssh-clients
Requires: 		systemtap-runtime = %{version}-%{release}
%if %{with_mokutil}
Requires:		mokutil
%endif

%description client
This package contains/requires the components needed to develop
systemtap scripts, and compile them using a local systemtap-devel
or a remote systemtap-server installation, then run them using a
local or remote systemtap-runtime.  It includes script samples and
documentation, and a copy of the tapset library for reference.

%package initscript
Summary:		Systemtap Initscripts
License: 		GPLv2+
Requires:		systemd
Requires: 		systemtap = %{version}-%{release}

%description initscript
This package includes a SysVinit script to launch selected systemtap
scripts at system startup, along with a dracut module for early
boot-time probing if supported.

%package exporter
Summary:		Systemtap-prometheus interoperation mechanism
License: 		GPLv2+
Requires:		systemtap-runtime = %{version}-%{release}

%description exporter
This package includes files for a systemd service that manages
systemtap sessions and relays prometheus metrics from the sessions
to remote requesters on demand.

%package runtime
Summary:		Runtime for executing a systemtap script
License: 		GPLv2+
Requires(pre):	shadow-utils

%description runtime
SystemTap runtime contains the components needed to execute
a systemtap script that was already compiled into a module
using a local or remote systemtap-devel installation.

%package devel
Summary:		Development headers and tools for compiling systemtap script
License: 		GPLv2+
Requires:		gcc make
Requires: 		elfutils-devel >= 0.179
Recommends:		(kernel-debug-devel if kernel-debug)
Recommends: 	(kernel-devel if kernel)

%description devel
This package contains the components needed to compile a systemtap
script from source form into executable (.ko) forms.  It may be
installed on a self-contained developer workstation (along with the
systemtap-client and systemtap-runtime packages), or on a dedicated
remote server (alongside the systemtap-server package).  It includes
a copy of the standard tapset library and the runtime library C files.

%package sdt-devel
Summary:		Static probe support tools
License: 		GPLv2+ and Public Domain
Requires:		python3-pyparsing

%description sdt-devel
This package includes the <sys/sdt.h> header file used for static
instrumentation compiled into userspace programs and libraries, along
with the optional dtrace-compatibility preprocessor to process related
.d files into tracing-macro-laden .h headers.

%if %{with testsuite}
%package testsuite
Summary:		Systemtap Testsuite
License: 		GPLv2+
Requires:		systemtap = %{version}-%{release}
Requires: 		systemtap-sdt-devel = %{version}-%{release}
Requires: 		systemtap-server = %{version}-%{release}
Requires: 		gcc gcc-c++ make 
Requires: 		dejagnu which elfutils grep /usr/bin/nc
Requires: 		glibc-devel strace avahi stress perf
%if %{with_crash}
Requires:		crash
%endif
Requires:		systemtap-runtime-python3 = %{version}-%{release}

%description testsuite
This package includes the dejagnu-based systemtap stress self-testing
suite.  This may be used by system administrators to thoroughly check
systemtap on the current system.
%endif

%package runtime-python3
Summary:		Systemtap Python 3 Runtime Support
License:		GPLv2+
Requires:		systemtap-runtime = %{version}-%{release}

%description runtime-python3
This package includes support files needed to run systemtap scripts
that probe python 3 processes.

%prep
%setup -q

%build
%configure \
    %{dyninst_config} \
    %{crash_config} \
    --enable-sqlite \
    --enable-docs=prebuilt \
    --with-rpm \
    --without-java \
    --disable-virt \
    --with-dracutstap=%{dracutstap} \
    --with-dracutbindir=%{dracutbindir} \
    --with-python3 \
    --without-python2-probes \
    --with-python3-probes \
    --disable-httpd \
    --with-bpf \
    --disable-silent-rules \
    --with-extra-version="rpm %{version}-%{release}"
%make_build

%install
%make_install
%find_lang %{name}
for dir in $(ls -1d $RPM_BUILD_ROOT%{_mandir}/{??,??_??}) ; do
    dir=$(echo $dir | sed -e "s|^$RPM_BUILD_ROOT||")
    lang=$(basename $dir)
    echo "%%lang($lang) $dir/man*/*" >> %{name}.lang
done

mkdir -p %{buildroot}%{_sysusersdir}
echo '%{systemtap_runtime_sysusers}' > %{buildroot}%{_sysusersdir}/systemtap-runtime.conf
echo '%{systemtap_server_sysusers}' > %{buildroot}%{_sysusersdir}/systemtap-server.conf

ln -s %{_datadir}/systemtap/examples
find $RPM_BUILD_ROOT%{_datadir}/systemtap/examples -type f -name '*.stp' -print0 | xargs -0 sed -i -r -e '1s@^#!.+stap@#!%{_bindir}/stap@'

chmod 755 $RPM_BUILD_ROOT%{_bindir}/staprun
install -c -m 755 stap-prep $RPM_BUILD_ROOT%{_bindir}/stap-prep
%if %{with testsuite}
echo '%{systemtap_testsuite_sysusers}' > %{buildroot}%{_sysusersdir}/systemtap-testsuite.conf
find testsuite -type f -name '.gitignore' -print0 | xargs -0 rm -f
cp -rp testsuite $RPM_BUILD_ROOT%{_datadir}/systemtap
%endif

install -D -m 644 macros.systemtap $RPM_BUILD_ROOT%{_rpmmacrodir}/macros.systemtap
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/stap-server
mkdir -p $RPM_BUILD_ROOT%{_localstatedir}/lib/stap-server
mkdir -p $RPM_BUILD_ROOT%{_localstatedir}/lib/stap-server/.systemtap
mkdir -p $RPM_BUILD_ROOT%{_localstatedir}/log/stap-server
touch $RPM_BUILD_ROOT%{_localstatedir}/log/stap-server/log
mkdir -p $RPM_BUILD_ROOT%{_localstatedir}/cache/systemtap
mkdir -p $RPM_BUILD_ROOT%{_localstatedir}/run/systemtap
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/logrotate.d
install -m 644 initscript/logrotate.stap-server $RPM_BUILD_ROOT%{_sysconfdir}/logrotate.d/stap-server

mkdir -p $RPM_BUILD_ROOT%{_unitdir}
touch $RPM_BUILD_ROOT%{_unitdir}/systemtap.service
mkdir -p $RPM_BUILD_ROOT%{_presetdir}
echo 'enable systemtap.service' > $RPM_BUILD_ROOT%{_presetdir}/42-systemtap.preset

install -m 644 initscript/systemtap.service $RPM_BUILD_ROOT%{_unitdir}/systemtap.service
mkdir -p $RPM_BUILD_ROOT%{_sbindir}
install -m 755 initscript/systemtap $RPM_BUILD_ROOT%{_sbindir}/systemtap-service

mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/systemtap
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/systemtap/conf.d
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/systemtap/script.d
install -m 644 initscript/config.systemtap $RPM_BUILD_ROOT%{_sysconfdir}/systemtap/config

mkdir -p $RPM_BUILD_ROOT%{_unitdir}
touch $RPM_BUILD_ROOT%{_unitdir}/stap-server.service
install -m 644 stap-server.service $RPM_BUILD_ROOT%{_unitdir}/stap-server.service
mkdir -p $RPM_BUILD_ROOT%{_tmpfilesdir}
install -m 644 stap-server.conf $RPM_BUILD_ROOT%{_tmpfilesdir}/stap-server.conf

mkdir -p $RPM_BUILD_ROOT%{_emacs_sitelispdir}
install -p -m 644 emacs/systemtap-mode.el* $RPM_BUILD_ROOT%{_emacs_sitelispdir}
mkdir -p $RPM_BUILD_ROOT%{_emacs_sitestartdir}
install -p -m 644 emacs/systemtap-init.el $RPM_BUILD_ROOT%{_emacs_sitestartdir}/systemtap-init.el
for subdir in ftdetect ftplugin indent syntax
do
    mkdir -p $RPM_BUILD_ROOT%{_datadir}/vim/vimfiles/$subdir
    install -p -m 644 vim/$subdir/*.vim $RPM_BUILD_ROOT%{_datadir}/vim/vimfiles/$subdir
done

mkdir -p $RPM_BUILD_ROOT%{dracutstap}
install -p -m 755 initscript/99stap/module-setup.sh $RPM_BUILD_ROOT%{dracutstap}
install -p -m 755 initscript/99stap/install $RPM_BUILD_ROOT%{dracutstap}
install -p -m 755 initscript/99stap/check $RPM_BUILD_ROOT%{dracutstap}
install -p -m 755 initscript/99stap/start-staprun.sh $RPM_BUILD_ROOT%{dracutstap}
touch $RPM_BUILD_ROOT%{dracutstap}/params.conf
pathfix.py -pni "%{__python3} %{py3_shbang_opts}" %{buildroot}%{python3_sitearch} %{buildroot}%{_bindir}/*

find $RPM_BUILD_ROOT%{_datadir} -type f -name '*.pdf' -delete

%post
(make -C %{_datadir}/systemtap/runtime/uprobes clean) >/dev/null 2>&1 || true
(/sbin/rmmod uprobes) >/dev/null 2>&1 || true

%preun
(make -C %{_datadir}/systemtap/runtime/uprobes clean) >/dev/null 2>&1 || true
(/sbin/rmmod uprobes) >/dev/null 2>&1 || true

%pre server
echo '%{systemtap_server_sysusers}' | systemd-sysusers --replace=%{_sysusersdir}/systemtap-server.conf -
exit 0

%post server
test -e ~stap-server && chmod 750 ~stap-server

if [ ! -f ~stap-server/.systemtap/rc ]; then
  mkdir -p ~stap-server/.systemtap
  chown stap-server:stap-server ~stap-server/.systemtap
  numcpu=`/usr/bin/getconf _NPROCESSORS_ONLN`
  if [ -z "$numcpu" -o "$numcpu" -lt 1 ]; then numcpu=1; fi
  nproc=`expr $numcpu \* 30`
  echo "--rlimit-as=614400000 --rlimit-cpu=60 --rlimit-nproc=$nproc --rlimit-stack=1024000 --rlimit-fsize=51200000" > ~stap-server/.systemtap/rc
  chown stap-server:stap-server ~stap-server/.systemtap/rc
fi

test -e %{_localstatedir}/log/stap-server/log || {
     touch %{_localstatedir}/log/stap-server/log
     chmod 644 %{_localstatedir}/log/stap-server/log
     chown stap-server:stap-server %{_localstatedir}/log/stap-server/log
}
/bin/systemd-tmpfiles --create %{_tmpfilesdir}/stap-server.conf >/dev/null 2>&1 || :
exit 0

%preun server
if [ $1 = 0 ] ; then
    /bin/systemctl --no-reload disable stap-server.service >/dev/null 2>&1 || :
    /bin/systemctl stop stap-server.service >/dev/null 2>&1 || :
fi
exit 0

%postun server
if [ "$1" -ge "1" ] ; then
    /bin/systemctl condrestart stap-server.service >/dev/null 2>&1 || :
fi
exit 0

%preun initscript
if [ $1 = 0 ] ; then
    /bin/systemctl --no-reload disable systemtap.service >/dev/null 2>&1 || :
    /bin/systemctl stop systemtap.service >/dev/null 2>&1 || :
fi
exit 0

%postun initscript
if [ "$1" -ge "1" ] ; then
    /bin/systemctl condrestart systemtap.service >/dev/null 2>&1 || :
fi
exit 0

%preun exporter
if [ $1 = 0 ] ; then
  /bin/systemctl stop stap-exporter.service >/dev/null 2>&1 || :
  /bin/systemctl disable stap-exporter.service >/dev/null 2>&1 || :
fi
exit 0

%postun exporter
if [ "$1" -ge "1" ]; then
   /bin/systemctl condrestart stap-exporter >/dev/null 2>&1 || :
fi
exit 0

%pre runtime
echo '%{systemtap_runtime_sysusers}' | systemd-sysusers --replace=%{_sysusersdir}/systemtap-runtime.conf -
exit 0

%if %{with testsuite}
%pre testsuite
echo '%{systemtap_testsuite_sysusers}' | systemd-sysusers --replace=%{_sysusersdir}/systemtap-testsuite.conf -
exit 0
%endif

%triggerin client -- systemtap-server
if test -e ~stap-server/.systemtap/ssl/server/stap.cert; then
   %{_libexecdir}/systemtap/stap-authorize-cert ~stap-server/.systemtap/ssl/server/stap.cert %{_sysconfdir}/systemtap/ssl/client >/dev/null
   %{_libexecdir}/systemtap/stap-authorize-cert ~stap-server/.systemtap/ssl/server/stap.cert %{_sysconfdir}/systemtap/staprun >/dev/null
fi
exit 0

%files

%files server -f systemtap.lang
%license COPYING
%doc README README.unprivileged AUTHORS NEWS 
%{_unitdir}/stap-server.service
%{_tmpfilesdir}/stap-server.conf
%config(noreplace) %{_sysconfdir}/logrotate.d/stap-server
%dir %{_sysconfdir}/stap-server
%dir %attr(0750,stap-server,stap-server) %{_localstatedir}/lib/stap-server
%dir %attr(0700,stap-server,stap-server) %{_localstatedir}/lib/stap-server/.systemtap
%dir %attr(0755,stap-server,stap-server) %{_localstatedir}/log/stap-server
%ghost %config(noreplace) %attr(0644,stap-server,stap-server) %{_localstatedir}/log/stap-server/log
%ghost %attr(0755,stap-server,stap-server) %{_localstatedir}/run/stap-server
%{_sysusersdir}/systemtap-server.conf
%{_bindir}/stap-server
%dir %{_libexecdir}/systemtap
%{_libexecdir}/systemtap/stap-serverd
%{_libexecdir}/systemtap/stap-start-server
%{_libexecdir}/systemtap/stap-stop-server
%{_libexecdir}/systemtap/stap-gen-cert
%{_libexecdir}/systemtap/stap-sign-module
%{_libexecdir}/systemtap/stap-authorize-cert
%{_libexecdir}/systemtap/stap-env
%{_mandir}/man7/error*
%{_mandir}/man7/stappaths.7*
%{_mandir}/man7/warning*
%{_mandir}/man8/stap-server.8*

%files client -f systemtap.lang
%license COPYING
%doc README README.unprivileged AUTHORS NEWS
%dir %{_datadir}/systemtap
%{_datadir}/systemtap/examples
%{_datadir}/systemtap/tapset
%{_bindir}/stap
%{_bindir}/stap-prep
%{_bindir}/stap-report
%{_mandir}/man1/stap.1*
%{_mandir}/man1/stap-prep.1*
%{_mandir}/man1/stap-merge.1*
%{_mandir}/man1/stap-report.1*
%{_mandir}/man1/stapref.1*
%{_mandir}/man3/*
%{_mandir}/man7/error*
%{_mandir}/man7/stappaths.7*
%{_mandir}/man7/warning*

%files initscript
%{_presetdir}/42-systemtap.preset
%{_unitdir}/systemtap.service
%dir %{_sysconfdir}/systemtap
%dir %{_sysconfdir}/systemtap/conf.d
%dir %{_sysconfdir}/systemtap/script.d
%config(noreplace) %{_sysconfdir}/systemtap/config
%dir %{_localstatedir}/cache/systemtap
%ghost %{_localstatedir}/run/systemtap
%dir %{dracutstap}
%{dracutstap}/*
%{_sbindir}/systemtap-service
%{_mandir}/man8/systemtap-service.8*

%files exporter
%{_sysconfdir}/stap-exporter
%{_sysconfdir}/sysconfig/stap-exporter
%{_unitdir}/stap-exporter.service
%{_sbindir}/stap-exporter
%{_mandir}/man8/stap-exporter.8*

%files runtime -f systemtap.lang
%license COPYING
%doc README README.security AUTHORS NEWS 
%{_sysusersdir}/systemtap-runtime.conf
%attr(4110,root,stapusr) %{_bindir}/staprun
%{_bindir}/stapsh
%{_bindir}/stap-merge
%{_bindir}/stap-report
%if %{with_dyninst}
%{_bindir}/stapdyn
%endif
%{_bindir}/stapbpf
%dir %{_libexecdir}/systemtap
%{_libexecdir}/systemtap/stapio
%{_libexecdir}/systemtap/stap-authorize-cert
%if %{with_crash}
%dir %{_libdir}/systemtap
%{_libdir}/systemtap/staplog.so*
%endif
%{_mandir}/man1/stap-report.1*
%{_mandir}/man7/error*
%{_mandir}/man7/stappaths.7*
%{_mandir}/man7/warning*
%{_mandir}/man8/stapsh.8*
%{_mandir}/man8/staprun.8*
%if %{with_dyninst}
%{_mandir}/man8/stapdyn.8*
%endif
%{_mandir}/man8/stapbpf.8*

%files devel -f systemtap.lang
%license COPYING
%doc README README.unprivileged AUTHORS NEWS 
%dir %{_datadir}/systemtap
%{_datadir}/systemtap/runtime
%{_datadir}/systemtap/tapset
%{_datadir}/vim/vimfiles/*/*.vim
%{_emacs_sitelispdir}/*.el*
%{_emacs_sitestartdir}/systemtap-init.el
%{_bindir}/stap
%{_bindir}/stap-prep
%{_bindir}/stap-profile-annotate
%{_bindir}/stap-report
%{_mandir}/man1/stap.1*
%{_mandir}/man1/stap-prep.1*
%{_mandir}/man1/stap-report.1*
%{_mandir}/man7/error*
%{_mandir}/man7/stappaths.7*
%{_mandir}/man7/warning*
%dir %{_libexecdir}/systemtap
%{_libexecdir}/systemtap/python/stap-resolve-module-function.py

%files sdt-devel
%license COPYING
%doc README AUTHORS NEWS 
%{_includedir}/sys/sdt.h
%{_includedir}/sys/sdt-config.h
%{_rpmmacrodir}/macros.systemtap
%{_bindir}/dtrace
%{_mandir}/man1/dtrace.1*

%if %{with testsuite}
%files testsuite
%dir %{_datadir}/systemtap
%{_datadir}/systemtap/testsuite
%{_sysusersdir}/systemtap-testsuite.conf
%endif

%files runtime-python3
%{python3_sitearch}/HelperSDT
%{python3_sitearch}/HelperSDT-*.egg-info

%changelog
* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 4.7-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 4.7-2
- Rebuilt for OpenCloudOS Stream 23

* Wed May 18 2022 Xiaojie Chen <jackxjchen@tencent.com> - 4.7-1
- Initial build
